#!/usr/bin/python
"""
Usage: di.py fetch [-C <cache-dir>] [-f remote|git|local] <info-url> <info-file>...
       di.py dump [-y <out.yaml>] [-j <out.json>] [-C <cache-dir>] [-f remote|git|local] <info-url> <info-file>...
       di.py --help | --version

Fetch, parse and dump remote distroinfo metadata.

Commands:
  fetch        fetch specified info into <cache-dir>
  dump         dump parsed info as YAML and/or JSON

Arguments:
  <info-url>   distroinfo repo URL
  <info-file>  info file(s) to parse

Options:
  -f, --fetcher remote|git|local
                             choose a distroinfo fetcher to use
  -y, --yaml-out <out.yaml>  dump parsed info into specified YAML file
  -j, --json-out <out.json>  dump parsed info into specified YAML file
  -C, --cache-dir <dir>      directory to store cached distroinfo metadata
  --version                  show distroinfo version
  -h, --help                 show usage help
"""  # noqa
# NOTE: docopt module is required to run this
# -*- encoding: utf-8 -*-
from __future__ import print_function
from docopt import docopt
import json
from timeit import default_timer as timer
import sys
import yaml

from distroinfo import __version__
from distroinfo import exception
from distroinfo.info import DistroInfo


def get_distroinfo(info_url, info_files, fetcher=None, cache_dir=None):
    kwargs = {}
    # use desired distroinfo fetcher
    if fetcher == 'local':
        kwargs['local_info'] = info_url
    elif fetcher == 'git':
        kwargs['remote_git_info'] = info_url
    else:
        kwargs['remote_info'] = info_url

    if cache_dir:
        kwargs['cache_base_path'] = cache_dir

    return DistroInfo(info_files, **kwargs)


def fetch(info_url, info_files, fetcher=None, cache_dir=None):
    if fetcher:
        fetch_str = ' %s' % fetcher
    else:
        fetch_str = ''
    print("Starting%s info fetch: %s" % (fetch_str, info_url))
    di = get_distroinfo(info_url, info_files,
                        fetcher=fetcher, cache_dir=cache_dir)
    cache_path = getattr(di.fetcher, 'cache_path', None)
    if cache_path:
        print("Cache: %s" % cache_path)
    t_start = timer()
    info = di.get_info()
    t_end = timer()
    t = t_end - t_start
    print("Fetched and parsed info with %d packages in %.2f s."
          % (len(info['packages']), t))
    return 0


def dump(info_url, info_files, fetcher=None, cache_dir=None,
         yaml_out=None, json_out=None):
    di = get_distroinfo(info_url, info_files,
                        fetcher=fetcher, cache_dir=cache_dir)
    info = di.get_info()
    if yaml_out or json_out:
        if yaml_out:
            print("Dumping YAML to: %s" % yaml_out)
            f = open(yaml_out, 'w')
            yaml.dump(info, f)
        if json_out:
            print("Dumping JSON to: %s" % json_out)
            f = open(json_out, 'w')
            json.dump(info, f)
    else:
        print(yaml.dump(info))
    return 0


def distroinfo(cargs, version=__version__):
    """
    distroinfo Command-Line Interface

    """
    code = 1
    args = docopt(__doc__, argv=cargs)
    try:
        if args['--version']:
            if not version:
                version = 'N/A'
            print(version)
            code = 0
        elif args['fetch']:
            code = fetch(
                info_url=args['<info-url>'],
                info_files=args['<info-file>'],
                cache_dir=args['--cache-dir'],
                fetcher=args['--fetcher'],
            )
        elif args['dump']:
            code = dump(
                info_url=args['<info-url>'],
                info_files=args['<info-file>'],
                yaml_out=args['--yaml-out'],
                json_out=args['--json-out'],
                cache_dir=args['--cache-dir'],
                fetcher=args['--fetcher'],
            )
    except (
            exception.InvalidInfoFormat,
            KeyboardInterrupt,
    ) as ex:
        code = getattr(ex, 'exit_code', code)
        print("")
        print(str(ex) or type(ex).__name__)

    return code


def main():
    """
    distroinfo CLI entry point
    """
    sys.exit(distroinfo(sys.argv[1:]))


if __name__ == '__main__':
    main()



from distroinfo import exception
from distroinfo.info import DistroInfo
import pytest

import tests.test_common as common


def test_invalid_info_file():
    di = DistroInfo('wololo.yml',
                    local_info=common.get_test_info_path('broken'))
    with pytest.raises(IOError):
        info = di.get_info()


def test_invalid_import():
    di = DistroInfo('invalid-import.yml',
                    local_info=common.get_test_info_path('broken'))
    with pytest.raises(IOError):
        info = di.get_info()


def test_circular_import():
    di = DistroInfo('circle1.yml',
                    local_info=common.get_test_info_path('broken'))
    with pytest.raises(exception.CircularInfoInclude):
        info = di.get_info()


def test_self_import():
    di = DistroInfo('self.yml',
                    local_info=common.get_test_info_path('broken'))
    with pytest.raises(exception.CircularInfoInclude):
        info = di.get_info()



import copy
from distroinfo.info import DistroInfo
from distroinfo import query

import tests.test_common as common


def test_attr_diff_base():
    di = DistroInfo('rdo.yml',
                    local_info=common.get_test_info_path('rdoinfo'))
    info = di.get_info()
    info2 = copy.deepcopy(info)
    # Update distgit for nova
    for pkg in info2['packages']:
        if pkg['project'] == 'nova':
            pkg['upstream'] = 'https://opendev.org/openstack/foo'

    diff = query.attr_diff(info, info2, 'upstream')
    assert(len(diff) == 1)
    assert(diff[0][0] == 'openstack-nova')
    assert(diff[0][1] == 'https://opendev.org/openstack/foo')


def test_attr_diff_nochanges():
    di = DistroInfo('rdo.yml',
                    local_info=common.get_test_info_path('rdoinfo'))
    info = di.get_info()
    info2 = copy.deepcopy(info)
    diff = query.attr_diff(info, info2, 'upstream')
    assert(len(diff) == 0)


def test_attr_diff_nosuchattr():
    di = DistroInfo('rdo.yml',
                    local_info=common.get_test_info_path('rdoinfo'))
    info = di.get_info()
    info2 = copy.deepcopy(info)
    # Update distgit for nova
    for pkg in info2['packages']:
        if pkg['project'] == 'nova':
            pkg['upstream'] = 'https://opendev.org/openstack/foo'

    diff = query.attr_diff(info, info2, 'fooattr')
    assert(len(diff) == 0)


def test_attr_diff_newpkg():
    di = DistroInfo('rdo.yml',
                    local_info=common.get_test_info_path('rdoinfo'))
    info = di.get_info()
    info2 = copy.deepcopy(info)
    # Add package
    newpkg = {'project': 'newproject',
              'name': 'openstack-newproject',
              'master-distgit': 'https://github.com/rdo-packages/new-distgit',
              'upstream': 'https://opendev.org/openstack/newproject'}
    info2['packages'].append(newpkg)

    diff = query.attr_diff(info, info2, 'master-distgit')
    assert(len(diff) == 1)
    assert(diff[0][0] == 'openstack-newproject')
    assert(diff[0][1] == 'https://github.com/rdo-packages/new-distgit')


def test_attr_diff_2_diffs():
    di = DistroInfo('rdo.yml',
                    local_info=common.get_test_info_path('rdoinfo'))
    info = di.get_info()
    info2 = copy.deepcopy(info)
    # Update distgit for nova
    for pkg in info2['packages']:
        if pkg['project'] == 'nova':
            pkg['upstream'] = 'https://opendev.org/openstack/foo'
        if pkg['project'] == 'cinder':
            pkg['upstream'] = 'https://opendev.org/openstack/foo-cinder'

    diff = query.attr_diff(info, info2, 'upstream')
    assert(len(diff) == 2)


def test_find_element_ok():
    di = DistroInfo('minimal.yml',
                    local_info=common.get_test_info_path('minimal'))
    info = di.get_info()
    finding = query.find_element(info, 'queens',
                                 info_key='releases')

    assert(finding)
    assert(finding == info['releases'][1])


def test_find_element_not_found():
    di = DistroInfo('minimal.yml',
                    local_info=common.get_test_info_path('minimal'))
    info = di.get_info()
    finding = query.find_element(info, 'aabb',
                                 info_key='releases')
    assert(not finding)


def test_find_element_in_sub_dict_list():
    di = DistroInfo('minimal.yml',
                    local_info=common.get_test_info_path('minimal'))
    info = di.get_info()
    finding = query.find_element(info, 'cloud7-openstack-rocky-testing',
                                 info_key='releases')

    assert(finding)
    assert(finding == info['releases'][0])



# Copyright (c) 2019 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class DistroInfoException(Exception):
    msg_fmt = "An unknown error occurred"

    def __init__(self, msg=None, **kwargs):
        self.kwargs = kwargs
        if not msg:
            try:
                msg = self.msg_fmt % kwargs
            except Exception:
                msg = self.msg_fmt
        super(DistroInfoException, self).__init__(msg)


class InfoFetcherRequired(DistroInfoException):
    msg_fmt = "No info fetcher was selected."


class InvalidInfoFormat(DistroInfoException):
    msg_fmt = "Invalid info format."


class MissingRequiredSection(InvalidInfoFormat):
    msg_fmt = "Info is missing required section: %(section)s"


class MissingRequiredItem(InvalidInfoFormat):
    msg_fmt = "Required item missing: %(item)s"


class UndefinedPackageConfig(InvalidInfoFormat):
    msg_fmt = "Package config isn't defined: %(conf)s"


class SubstitutionFailed(InvalidInfoFormat):
    msg_fmt = "Substitution failed for string: %(txt)s"


class DuplicatedProject(InvalidInfoFormat):
    msg_fmt = "Duplicated project: %(prj)s"


class CircularInfoInclude(InvalidInfoFormat):
    msg_fmt = "Circular info include: %(info)s"


class InvalidQuery(DistroInfoException):
    msg_fmt = "Invalid query: %(why)s"


class InvalidPackageFilter(DistroInfoException):
    msg_fmt = "Invalid package filter: %(why)s"


class InvalidRemoteInfoRef(DistroInfoException):
    msg_fmt = ("Remote info referenced but not defined in 'remote-info' "
               "section: %(remote)s")


class RepoError(DistroInfoException):
    msg_fmt = "Repository error: %(what)s"


class NotADirectory(DistroInfoException):
    msg_fmt = "Not a directory: %(path)s"


class CommandNotFound(DistroInfoException):
    msg_fmt = "Command not found: %(cmd)s"


class CommandFailed(DistroInfoException):
    msg_fmt = "Command failed with return code %(code)s: %(cmd)s"


class RemoteFetchError(DistroInfoException):
    msg_fmt = "Failed to fetch remote file: %(code)s %(reason)s %(url)s"



#!/usr/bin/env python

import re
import setuptools
import sys

# In python < 2.7.4, a lazy loading of package `pbr` will break
# setuptools if some other modules registered functions in `atexit`.
# solution from: http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing  # noqa
except ImportError:
    pass


# Only require pytest_runner for setup when testing. This uses the
# recommendation from
# https://pypi.org/project/pytest-runner/#conditional-requirement
needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

setuptools.setup(
    setup_requires=['pbr'] + pytest_runner,
    tests_require=['pytest'],
    pbr=True)
